import sys
import typing


def bake():
    ''' Bake dynamic paint image sequence surface

    '''

    pass


def output_toggle(output: typing.Union[str, int] = 'A'):
    ''' Add or remove Dynamic Paint output data layer

    :param output: Output Toggle
    :type output: typing.Union[str, int]
    '''

    pass


def surface_slot_add():
    ''' Add a new Dynamic Paint surface slot

    '''

    pass


def surface_slot_remove():
    ''' Remove the selected surface slot

    '''

    pass


def type_toggle(type: typing.Union[str, int] = 'CANVAS'):
    ''' Toggle whether given type is active or not

    :param type: Type
    :type type: typing.Union[str, int]
    '''

    pass
